@@ -454,11 +454,11 @@ show_attributes( const FL_OBJECT * obj )
     int i,
         lstyle,
         spstyle,
-#if ! defined FL_ENABLE_XFT
+#if ! FL_ENABLE_XFT
         oksize,
 #endif
         align = fl_to_outside_lalign( obj->align );
-#if ! defined FL_ENABLE_XFT
+#if ! FL_ENABLE_XFT
     static char othersize[ 32 ];
 #endif
 
@@ -503,7 +503,7 @@ show_attributes( const FL_OBJECT * obj )
 
 	/* d) label font size */
 
-#if defined FL_ENABLE_XFT
+#if FL_ENABLE_XFT
     fl_set_spinner_value( xft_sizeobj, obj->lsize );
 #else
     for ( oksize = i = 0; ! oksize && i < ( int ) NFSIZE; i++ )
@@ -653,7 +653,7 @@ readback_attributes( FL_OBJECT * obj )
 /***************************************
  * Turns the string from the label input field into a string suitable for
  * an object label (taking care of newlines and shortcut markers) and
- * the sets the label of the object being edited.
+ * then sets the label of the object being edited.
  ***************************************/
 
 void
@@ -841,7 +841,7 @@ get_shortcut_string( const FL_OBJECT * obj )
 {
     static char tmps[ 127 ];
     char *p = tmps;
-    long *sc = obj->shortcut;
+    FL_Char *sc = obj->shortcut;
     int n;
 
     for ( *p = '\0'; sc && *sc; sc++ )
