@@ -561,13 +561,9 @@ handle_chart( FL_OBJECT * ob,
               int         event,
               FL_Coord    mx   FL_UNUSED_ARG,
               FL_Coord    my   FL_UNUSED_ARG,
-              int         key  FL_UNUSED_ARG,
+              FL_Char     key  FL_UNUSED_ARG,
               void      * ev   FL_UNUSED_ARG )
 {
-#if FL_DEBUG >= ML_DEBUG
-    M_info( "handle_chart", fli_event_name( event ) );
-#endif
-
     switch ( event )
     {
         case FL_DRAW:
