@@ -202,9 +202,10 @@ void fli_scale_form( FL_FORM *,
                      double,
                      double );
 
-void fli_handle_form( FL_FORM *,
-                      int, int,
-                      XEvent * );
+void fli_handle_form( FL_FORM * form,
+                      int       event,
+                      FL_Char   key,
+                      XEvent  * xevent );
 
 FL_OBJECT *fli_end_group( void );
 
@@ -212,7 +213,7 @@ void fli_handle_object( FL_OBJECT *,
                         int,
                         FL_Coord,
                         FL_Coord,
-                        int,
+                        FL_Char,
                         XEvent *,
                         int );
 
@@ -252,7 +253,7 @@ void fli_set_object_visibility( FL_OBJECT * obj,
 
 
 int fli_mouse_wheel_to_keypress( int       * ev,
-                                 int       * key,
+                                 FL_Char   * key,
                                  void      * xev );
 
 void fli_notify_object( FL_OBJECT * obj,
@@ -567,6 +568,7 @@ typedef struct fli_context_ {
     void               * xim;
     void               * xic;
 #endif
+    Window               xic_win;
     unsigned int         navigate_mask;     /* input field            */
     long                 reserverd[ 6 ];
 } FLI_CONTEXT;
@@ -636,7 +638,7 @@ void fli_watch_io( FLI_IO_REC *,
                    long );
 
 int fli_do_shortcut( FL_FORM *,
-                     int,
+                     FL_Char,
                      FL_Coord,
                      FL_Coord,
                      XEvent * );
@@ -780,7 +782,7 @@ void fli_hide_and_get_region( FL_OBJECT *,
                               Region    * );
 
 int fli_convert_shortcut( const char *,
-                          long * );
+                          FL_Char * );
 
 int fli_get_underline_pos( const char *,
                            const char * );
@@ -965,7 +967,7 @@ int fli_is_tooltip_form( FL_FORM * );
 void fli_do_radio_push( FL_OBJECT *,
                         FL_Coord,
                         FL_Coord,
-                        int,
+                        FL_Char,
                         void *,
                         int );
 
@@ -1120,6 +1122,18 @@ int fli_is_valid_dir( const char * name );
 int fli_font_index_compare( const void * arg1,
                             const void * arg2 );
 
+FL_FONT * fli_find_font_by_index( FL_FONT * base,
+                                  size_t    cnt,
+                                  size_t    index );
+
+int fli_font_size_compare( const void * arg1,
+                           const void * arg2 );
+
+FL_SIZED_FONT *
+fli_search_sized_fonts( FL_SIZED_FONT * base,
+						size_t          cnt,
+						int             size );
+
 char * fli_convert_to_full_xlfd( const char * pattern );
 
 char * fli_get_fname( const FL_FONT * font,
@@ -1157,6 +1171,8 @@ void * fli_remove_and_resize( void   * arr,
                               size_t   index,
                               size_t   remove_cnt );
 
+#include "utf8.h"
+
 #endif /* ! defined FL_INTERNAL_H */
 
 
