@@ -38,10 +38,8 @@
 #include <X11/Xresource.h>
 #include <ctype.h>
 #include <sys/types.h>
-
-#if 0
 #include <locale.h>
-#endif
+#include <X11/Xatom.h>
 
 #ifdef FL_WIN32
 #include <X11/Xw32defs.h>
@@ -897,6 +895,158 @@ fli_set_input_navigate( unsigned int mask )
 }
 
 
+#ifdef XlibSpecificationRelease
+
+/***************************************
+ * This function chooses the "more desirable" of two input styles.  The
+ * style with the more complicated Preedit style is returned, and if the
+ * styles have the same Preedit styles, then the style with the more
+ * complicated Status style is returned.  There is no "official" way to
+ * order interaction styles; this one seems reasonable, though.
+ * This is a long procedure for a simple heuristic.
+ *
+ * "Stolen" from the XLib manual.
+ ***************************************/
+
+static XIMStyle
+ChooseBetterStyle( XIMStyle style1,
+                   XIMStyle style2 )
+{
+    XIMStyle s, t;
+    XIMStyle preedit =   XIMPreeditArea | XIMPreeditCallbacks
+                       | XIMPreeditPosition | XIMPreeditNothing
+                       | XIMPreeditNone;
+    XIMStyle status  =   XIMStatusArea | XIMStatusCallbacks
+                       | XIMStatusNothing | XIMStatusNone;
+
+    if ( style1 == 0 )
+        return style2;
+
+    if ( style2 == 0 )
+        return style1;
+
+    if ( ( style1 & ( preedit | status ) ) ==
+                                        ( style2 & ( preedit | status ) ) )
+        return style1;
+
+    s = style1 & preedit;
+    t = style2 & preedit;
+
+    if ( s != t )
+    {
+        if ( s | t | XIMPreeditCallbacks )
+            return s == XIMPreeditCallbacks ? style1 : style2;
+        else if ( s | t | XIMPreeditPosition )
+            return s == XIMPreeditPosition ? style1 : style2;
+        else if ( s | t | XIMPreeditArea )
+            return (s == XIMPreeditArea)?style1:style2;
+        else if (s | t | XIMPreeditNothing)
+            return s == XIMPreeditNothing ? style1 : style2;
+    }
+    else
+    {
+        /* if preedit flags are the same, compare status flags */
+
+        s = style1 & status;
+        t = style2 & status;
+
+        if ( s | t | XIMStatusCallbacks )
+            return s == XIMStatusCallbacks ? style1 : style2;
+        else if ( s | t | XIMStatusArea )
+            return s == XIMStatusArea ? style1 : style2;
+        else if ( s | t | XIMStatusNothing )
+            return s == XIMStatusNothing ? style1 : style2;
+    }
+}
+
+
+/***************************************
+ * Function to set up an IM ("input method") and an IC ("input context")
+ * to be used all over the library
+ ***************************************/
+
+static void
+setup_im_and_ic( void )
+{
+    XIMStyle best_style;
+    XIMStyles * im_supported_styles;
+    XIMStyle app_supported_styles;
+    int i;
+
+    if (    ! setlocale( LC_ALL, "" )
+         || !  XSupportsLocale( )
+         || ! XSetLocaleModifiers( "" ) )
+        return;
+
+    /* Use the same input method throughout xforms */
+
+    fli_context->xim = XOpenIM( fl_display, NULL, NULL, NULL );
+
+    if ( ! fli_context->xim )
+    {
+        M_err( "fl_initialize", "Could not create an input method" );
+        return;
+    }
+
+    /* And also the same input context. Start with figuring out which
+       styles the IM can support */
+            
+    XGetIMValues( fli_context->xim, XNQueryInputStyle,
+                  &im_supported_styles, NULL );
+
+    /* Set flags for the styles the library can support */
+            
+    app_supported_styles =   XIMPreeditNone
+                           | XIMPreeditNothing
+                           | XIMPreeditArea
+                           | XIMStatusNone
+                           | XIMStatusNothing
+                           | XIMStatusArea;
+
+    /* Now look at each of the IM supported styles, and chose the
+     * "best" one that we can support. */
+
+    best_style = 0;
+    for ( i = 0; i < im_supported_styles->count_styles; i++ )
+    {
+        XIMStyle style = im_supported_styles->supported_styles[ i ];
+
+        if ( ( style & app_supported_styles ) == style )
+            best_style = ChooseBetterStyle( style, best_style );
+    }
+
+    XFree( im_supported_styles );
+
+    /* If we couldn't support any of them print an error message */
+
+    if ( best_style == 0 )
+    {
+        M_err( "fl_intialize", "Can't find a fitting IC style" );
+        XCloseIM( fli_context->xim );
+        fli_context->xim = NULL;
+        return;
+    }
+
+    /* Try to create the input context */
+
+    fli_context->xic_win = None;
+    fli_context->xic = XCreateIC( fli_context->xim,
+                                  XNInputStyle, best_style,
+                                  XNClientWindow, fl_root,
+                                  ( char * ) NULL );
+
+    if ( ! fli_context->xic )
+    {
+        M_err( "fl_initialize",
+               "Could not create an input context" );
+        XCloseIM( fli_context->xim );
+        fli_context->xim = NULL;
+    }
+}
+
+#endif
+
+
 /***************************************
  ***************************************/
 
@@ -958,7 +1108,7 @@ fl_initialize( int        * na,
 
     XrmParseCommand( &cmddb, copt, Ncopt, fl_app_name, na, arg );
 
-    /* if there are still more left and  appopt is not zero */
+    /* If there are still more left and appopt is not zero */
 
     if ( appopt && na && *na )
         XrmParseCommand( &cmddb, appopt, nappopt,
@@ -989,8 +1139,8 @@ fl_initialize( int        * na,
 
     if ( ! ( fl_display = XOpenDisplay( buf ) ) )
     {
-        /* if no display is set, there is no guarantee that buf
-           is long enough to contain the DISPLAY setting */
+        /* if no display is set there is no guarantee that 'buf' is long
+           enough to contain the DISPLAY setting */
 
         M_err( "fl_initialize", "%s: Can't open display %s", fli_argv[ 0 ],
                XDisplayName( buf[ 0 ] ? buf : 0 ) );
@@ -1153,35 +1303,9 @@ fl_initialize( int        * na,
     fli_init_context( );
 
 #ifdef XlibSpecificationRelease
-    if ( XSupportsLocale( ) )
-    {
-        XSetLocaleModifiers( "" );
+    /* Create input method and input context */
 
-        /* Use the same input method throughout xforms */
-
-        fli_context->xim = XOpenIM( fl_display, NULL, NULL, NULL );
-
-        /* Also use the same input context */
-
-        if ( fli_context->xim )
-        {
-            int style =  XIMPreeditNothing | XIMStatusNothing;
-
-            fli_context->xic = XCreateIC( fli_context->xim,
-                                          XNInputStyle, style,
-                                          ( char * ) NULL );
-
-        /* Clean-up on failure */
-
-            if ( ! fli_context->xic )
-            {
-                M_err( "fl_initialize", "Could not create an input context" );
-                XCloseIM( fli_context->xim );
-            }
-        }
-        else
-            M_err( "fl_initialize", "Could not create an input method" );
-    }
+    setup_im_and_ic( );
 #endif
 
     fli_default_xswa( );
@@ -1341,7 +1465,7 @@ fl_finish( void )
     /* Get rid of memory for input methods */
 
 #ifdef XlibSpecificationRelease
-    if ( fli_context && XSupportsLocale( ) && fli_context->xim )
+    if ( fli_context && fli_context->xim )
     {
         if ( fli_context->xic )
             XDestroyIC( fli_context->xic );
@@ -1368,8 +1492,6 @@ fl_finish( void )
  * Find out about virtual root. Taken from XFaq
  ***************************************/
 
-#include <X11/Xatom.h>
-
 static int
 xerror_handler( Display     * d  FL_UNUSED_ARG,
                 XErrorEvent * xev )
