@@ -86,26 +86,34 @@ fl_input_end_return_handling( int type )
  ***************************************/
 
 static int
-fli_XLookupString( XKeyEvent * xkey,
+fli_XLookupString( XKeyEvent * xkev,
                    char      * buf,
                    int         buflen,
                    KeySym    * ks )
 {
-    int len = INT_MIN;
+    int len;
 
     if ( ! fli_context->xic )
-        len = XLookupString( xkey, buf, buflen, ks, 0 );
+        len = XLookupString( xkev, buf, buflen - 1, ks, NULL );
     else
     {
         Status status;
 
-        len = XmbLookupString( fli_context->xic, xkey, buf, buflen, ks,
+#if defined X_HAVE_UTF8_STRING
+        len = Xutf8LookupString( fli_context->xic, xkev, buf, buflen - 1, ks,
+                                 &status );
+#else
+        len = XmbLookupString( fli_context->xic, xkev, buf, buflen - 1, ks,
                                &status );
+#endif
 
         if ( status == XBufferOverflow )
-            len = -len;
+            len *= -1;
     }
 
+    if ( len >= 0 )
+        buf[ len ] = '\0';
+
     return len;
 }
 
@@ -145,7 +153,7 @@ void
 fli_do_radio_push( FL_OBJECT * obj,
                    FL_Coord    x,
                    FL_Coord    y,
-                   int         key,
+                   FL_Char     key,
                    void      * xev,
                    int         no_callbacks )
 {
@@ -192,15 +200,15 @@ fli_do_radio_push( FL_OBJECT * obj,
 
 static int
 do_shortcut( FL_FORM  * form,
-             int        key,
+             FL_Char    key,
              FL_Coord   x,
              FL_Coord   y,
              XEvent   * xev )
 {
-    int key1,
-        key2;
+    FL_Char key1,
+            key2;
     FL_OBJECT *obj;
-    long *s;
+    FL_Char *s;
 
     key1 = key2 = key;
 
@@ -208,18 +216,18 @@ do_shortcut( FL_FORM  * form,
 
     if ( fl_keypressed( XK_Alt_L ) || fl_keypressed( XK_Alt_R ) )
     {
-        if ( key < 256 )
+        if ( key <= 0x7F )
         {
-            key1 = FL_ALT_MASK + ( islower( ( int ) key ) ?
-                                   toupper( ( int ) key ) : key );
+            key1 = FL_ALT_MASK + toupper( key );
             key2 = FL_ALT_MASK + key;
         }
         else
             key1 = key2 = FL_ALT_MASK + key;
     }
 
-    M_info( "do_shortcut", "win = %ld key = %d %d %d",
-            form->window, key, key1, key2 );
+    M_info( "do_shortcut", "win = %ld key = %lu %lu %lu",
+            form->window, ( unsigned long ) key, ( unsigned long ) key1,
+            ( unsigned long ) key2 );
 
     /* Check if an object has this as a shortcut */
 
@@ -272,7 +280,7 @@ do_shortcut( FL_FORM  * form,
 
 int
 fli_do_shortcut( FL_FORM  * form,
-                 int        key,
+                 FL_Char    key,
                  FL_Coord   x,
                  FL_Coord   y,
                  XEvent   * xev )
@@ -296,7 +304,7 @@ fli_do_shortcut( FL_FORM  * form,
 
 static void
 handle_keyboard( FL_FORM  * form,
-                 int        key,
+                 FL_Char    key,
                  FL_Coord   x,
                  FL_Coord   y,
                  void     * xev )
@@ -304,7 +312,7 @@ handle_keyboard( FL_FORM  * form,
     FL_OBJECT *obj,
               *special;
 
-    /* Always check shortcut first */
+    /* Always check for shortcuts first */
 
     if ( fli_do_shortcut( form, key, x, y, xev ) )
         return;
@@ -329,7 +337,7 @@ handle_keyboard( FL_FORM  * form,
 
         /* Handle special keys first */
 
-        if ( key > 255 )
+        if ( utf8_get_char_bytes( key ) == -1 )
         {
             if (    IsLeft( key )
                  || IsRight( key )
@@ -359,13 +367,12 @@ handle_keyboard( FL_FORM  * form,
         /* The <Tab> and <Return> keys move the focus to the next or previous
            input object (depending on <SHIFT> being pressed also, and for
            <Return> only if the current focus object hasn't set FL_KEY_TAB as
-           it's the case for  multiline input objects) */
+           it's the case for multiline input objects). Don't do anything
+           if the form has only a single object that can have the focus. */
 
         if (    key == '\t'
              || ( key == '\r' && ! ( focusobj->wantkey & FL_KEY_TAB ) ) )
         {
-            fli_handle_object( focusobj, FL_UNFOCUS, x, y, 0, xev, 1 );
-
             if ( ( ( XKeyEvent * ) xev )->state & fli_context->navigate_mask )
             {
                 if ( ! ( obj = fli_find_object_backwards( focusobj->prev,
@@ -380,24 +387,29 @@ handle_keyboard( FL_FORM  * form,
                         
                     obj = fli_find_first( form, FLI_FIND_INPUT, 0, 0 );
             }
-                
-            fli_handle_object( obj, FL_FOCUS, x, y, 0, xev, 1 );
+
+            if ( focusobj != obj )
+            {
+                fli_handle_object( focusobj, FL_UNFOCUS, x, y, 0, xev, 1 );
+                fli_handle_object( obj, FL_FOCUS, x, y, 0, xev, 1 );
+            }
         }
         else if ( focusobj->wantkey != FL_KEY_SPECIAL )
             fli_handle_object( focusobj, FL_KEYPRESS, x, y, key, xev, 1 );
         return;
     }
 
-    /* Keyboard input is not wanted */
+    /* Bail out if keyboard input is not wanted */
 
     if ( ! special || special->wantkey == 0 )
         return;
 
     /* Space is an exception for browser */
 
-    if (   (    ( key > 255 || key == ' ' )
-             && special->wantkey & FL_KEY_SPECIAL )
-         || ( key < 255 && special->wantkey & FL_KEY_NORMAL )
+    if (    (    ( utf8_get_char_bytes( key ) == - 1|| key == ' ' )
+              && special->wantkey & FL_KEY_SPECIAL )
+         || (    utf8_get_char_bytes( key ) != -1
+              && special->wantkey & FL_KEY_NORMAL )
          || ( special->wantkey == FL_KEY_ALL ) )
         fli_handle_object( special, FL_KEYPRESS, x, y, key, xev, 1 );
 }
@@ -410,7 +422,7 @@ handle_keyboard( FL_FORM  * form,
 void
 fli_handle_form( FL_FORM * form,
                  int       event,
-                 int       key,
+                 FL_Char   key,
                  XEvent  * xev )
 {
     FL_OBJECT *obj = NULL;
@@ -675,6 +687,7 @@ do_interaction_step( int wait_io )
         fli_xevent_name( "MainLoop", &st_xev );
 #endif
 
+
     fli_compress_event( &st_xev, evform->compress_mask );
 
     fli_int.query_age++;
@@ -697,15 +710,14 @@ do_interaction_step( int wait_io )
             if ( evform->focusobj )
                 fli_int.keyform = evform;
 
-            if ( fli_context->xic )
-                XSetICValues( fli_context->xic,
-                              XNFocusWindow, st_xev.xfocus.window,
-                              XNClientWindow, st_xev.xfocus.window,
-                              ( char * ) NULL );
             break;
 
         case FocusOut:
             fli_int.keyform = NULL;
+#if defined XlibSpecificationRelease
+//            if ( fli_context->xic )
+//                XUnsetICFocus( fli_context->xic );
+#endif
             break;
 
         case KeyPress:
@@ -935,8 +947,9 @@ handle_keyboard_event( XEvent * xev,
 {
     Window win = xev->xkey.window;
     KeySym keysym = 0;
-    unsigned char keybuf[ 227 ];
-    int kbuflen;
+    static unsigned char * keybuf = NULL;
+    static int keybuflen = 0;
+    int len;
 
     fli_int.mousex    = xev->xkey.x;
     fli_int.mousey    = xev->xkey.y;
@@ -966,21 +979,31 @@ handle_keyboard_event( XEvent * xev,
     if ( ! fli_int.keyform )
         return;
 
-    kbuflen = fli_XLookupString( ( XKeyEvent * ) xev, ( char * ) keybuf,
-                                 sizeof keybuf, &keysym );
-
-    if ( kbuflen < 0 )
+    if ( ! keybuf )
     {
-        if ( kbuflen != INT_MIN )
-            M_err( "handle_keyboard_event", "keyboad buffer overflow?" );
-        else
-            M_err( "handle_keyboard_event", "fli_XLookupString failed?" );
-
-        return;
+        keybuf = fl_malloc( 5 );
+        keybuflen = 5;
     }
 
+    while ( 1 )
+    {
+        len = fli_XLookupString( ( XKeyEvent * ) xev, ( char * ) keybuf,
+                                 keybuflen, &keysym );
+
+        if ( len >= 0 )
+            break;
+
+        len *= -1;
+
+        M_warn( "handle_keyboard_event",
+                "Key with lenght longger than 4 (%d) encountered", len );
+
+        keybuf = fl_realloc( keybuf, len + 1 );
+        keybuflen = len + 1;
+    }
+            
     /* Ignore modifier keys as they don't cause action and are taken care
-       of by the lookupstring routine */
+       of by the XLookupString() function */
 
     if ( IsModifierKey( keysym ) )
         /* empty */ ;
@@ -990,16 +1013,23 @@ handle_keyboard_event( XEvent * xev,
 
         fli_handle_form( fli_int.keyform, formevent, '\t', xev );
     }
-    else if ( IsCursorKey( keysym ) || kbuflen == 0 )
+    else if ( IsCursorKey( keysym ) || len == 0 )
         fli_handle_form( fli_int.keyform, formevent, keysym, xev );
     else
     {
-        unsigned char *ch;
+        /* Note: with input methods more than one character may have
+           arrived with a single keyboard event */
 
-        /* All regular keys, including mapped strings */
+        const char * p = ( char * ) keybuf;
+        int cl = 0;
 
-        for ( ch = keybuf; ch < keybuf + kbuflen && fli_int.keyform; ch++ )
-            fli_handle_form( fli_int.keyform, formevent, *ch, xev );
+        do
+        {
+            fli_handle_form( fli_int.keyform, formevent,
+                             utf8_to_num( p + cl ), xev );
+            cl = utf8_get_byte_count( p );
+            cl *= cl >= 0 ? 1 : -1;
+        } while ( ( len -= cl ) > 0 );
     }
 }
 
@@ -1026,6 +1056,21 @@ handle_EnterNotify_event( FL_FORM * evform )
         fli_handle_form( fli_int.mouseform, FL_LEAVE,
                          xmask2button( fli_int.keymask ), &st_xev );
 
+#if defined XlibSpecificationRelease
+    /* If we've got aN input context and the window changed we've got
+       to tell it about the new window */
+
+    if ( fli_context->xic && fli_context->xic_win != win )
+    {
+        fli_context->xic_win = win;
+        XSetICValues( fli_context->xic,
+                      XNFocusWindow, win,
+                      XNClientWindow, win,
+                      ( char * ) NULL );
+        XSetICFocus( fli_context->xic );
+    }
+#endif
+
     if ( evform )
     {
         fli_int.mouseform = evform;
