@@ -331,7 +331,13 @@ draw_input( FL_OBJECT * obj )
 }
 
 
-#define DELIM( c )  ( c == ' ' || c == ',' || c == '.' || c == '\n' )
+#define DELIM_OLD( c )                                                       \
+    (    ( unsigned char ) ( c ) < 0x7F                                      \
+      && ( ( c ) == ' ' || ( c ) == ',' || ( c ) == '.' || ( c ) == '\n' ) )
+
+#define DELIM_NEW( c )                                                  \
+    (    ( unsigned char ) ( c ) < 0x7F                                 \
+      && ! ( isalnum( ( unsigned int ) ( c ) ) || ( c ) == '_' ) )
 
 
 /***************************************
@@ -378,34 +384,22 @@ handle_select( FL_Coord    mx,
 #if defined USE_CLASSIC_EDITKEYS
         if ( sp->str[ thepos ] == ' ' )
             return 0;
-
-        for ( n = thepos; sp->str[ n ] && ! DELIM( sp->str[ n ] ); n++ )
+        for ( n = thepos; sp->str[ n ] && ! DELIM_OLD( sp->str[ n ] ); n++ )
             /* empty */ ;
         sp->endrange = n;
 
-        for ( n = thepos; n >= 0 && ! DELIM( sp->str[ n ] ); n-- )
+        for ( n = thepos; n >= 0 && ! DELIM_OLD( sp->str[ n ] ); n-- )
             /* empty */ ;
 
         sp->beginrange = n + 1;
 #else
-        if (    ! isalnum( ( unsigned char ) sp->str[ thepos ] )
-             && sp->str[ thepos ] != '_' )
+        if ( DELIM_NEW( sp->str[ thepos ] ) )
         {
-            for ( n = thepos;
-                     sp->str[ n ]
-                  && ! isalnum( ( unsigned char ) sp->str[ n ] )
-                  && sp->str[ n ] != '_'
-                  && sp->str[ n ] != '\n';
-                  n++ )
+            for ( n = thepos; sp->str[ n ] && DELIM_NEW( sp->str[ n ] ); n++ )
                 /* empty */ ;
             sp->endrange = n;
 
-            for ( n = thepos;
-                     n
-                  && ! isalnum( ( unsigned char ) sp->str[ n ] )
-                  && sp->str[ n ] != '_'
-                  && sp->str[ n ] != '\n';
-                  n-- )
+            for ( n = thepos; n && DELIM_NEW( sp->str[ n ] ); n-- )
                 /* empty */ ;
             if ( n > 0 )
                 ++n;
@@ -413,18 +407,11 @@ handle_select( FL_Coord    mx,
         }
         else
         {
-            for ( n = thepos;
-                     sp->str[ n ]
-                  && (    isalnum( ( unsigned char ) sp->str[ n ] )
-                       || sp->str[ n ] == '_' );
-                  n++ )
+            for ( n = thepos; sp->str[ n ] && ! DELIM_NEW( sp->str[ n ] ); n++ )
                 /* empty */ ;
             sp->endrange = n;
 
-            for ( n = thepos;
-                  n && (    isalnum( ( unsigned char ) sp->str[ n ] )
-                         || sp->str[ n ] == '_' );
-                  n-- )
+            for ( n = thepos; n && ! DELIM_NEW(sp->str[ n ] ); n-- )
                 /* empty */ ;
             if ( n > 0 )
                 ++n;
@@ -493,24 +480,37 @@ static char cutbuf[ MAXCBLEN ];
 
 
 /***************************************
- * Delete a single char. dir =1 for next, dir -1 for prev
+ * Delete a single char, dir =1 for next, dir -1 for prev
  ***************************************/
 
 static void
 delete_char( FLI_INPUT_SPEC * sp,
-             int              dir,
-             int              slen )
+             int              dir )
 {
-    int i = sp->position - ( dir < 0 );
-
-    if ( sp->str[ i ] == '\n' )
+    if ( dir == 1 )
     {
-        sp->lines--;
-        sp->ypos -= dir < 0;
-    }
+        char * new_pos = utf8_next_char_pos( sp->str + sp->position );
 
-    memmove( sp->str + i, sp->str + i + 1, slen - i );
-    sp->position -= dir < 0;
+        if ( sp->str[ sp->position ] == '\n' )
+            sp->lines--;
+
+        memmove( sp->str + sp->position, new_pos, strlen( new_pos ) + 1 );
+    }
+    else
+    {
+        char * new_pos = utf8_prev_char_pos( sp->str + sp->position );
+
+        if ( *new_pos == '\n' )
+        {
+            sp->lines--;
+            sp->ypos--;
+        }
+
+        memmove( new_pos, sp->str + sp->position,
+                 strlen( sp->str + sp->position ) + 1 );
+
+        sp->position = new_pos - sp->str;
+    }
 }
 
 
@@ -528,8 +528,6 @@ delete_piece( FL_OBJECT * obj,
     memmove( sp->str + start, sp->str + end + 1, strlen( sp->str + end ) );
     sp->position = start;
 
-    /* This can be expensive TODO */
-
     sp->lines = fl_get_input_numberoflines( obj );
     fl_get_input_cursorpos( obj, &sp->xpos, &sp->ypos );
 }
@@ -540,8 +538,10 @@ delete_piece( FL_OBJECT * obj,
                          ( ( FLI_INPUT_SPEC * ) ( o )->spec )->str + ( b ),  \
                          ( e ) - ( b ) )
 
-#define IsRegular( k )  (    ( k ) == '\n'                                \
-                          || ( key >= 32 && key <= 255 && key != 127 ) )
+#define IsRegular( k )  (    ( k ) == '\n'                     \
+                          || (    key >= ' '                   \
+                               && utf8_get_char_bytes( k ) > 0 \
+                               && key != 0x7F ) )
 
 
 /***************************************
@@ -553,7 +553,7 @@ delete_piece( FL_OBJECT * obj,
 static FL_EditKeymap kmap;
 
 static int paste_it( FL_OBJECT *,
-                     const unsigned char *,
+                     const char *,
                      int );
 
 static void set_default_keymap( int );
@@ -567,7 +567,7 @@ static void set_default_keymap( int );
 
 static void
 handle_movement( FL_OBJECT * obj,
-                 int         key,
+                 FL_Char     key,
                  int         slen,
                  int         startpos,
                  int         kmask )
@@ -610,7 +610,14 @@ handle_movement( FL_OBJECT * obj,
         if ( shiftkey_down( kmask ) )
             sp->position = startpos;
         else if ( sp->position > 0 )
-            sp->position--;
+        {
+            sp->position -= utf8_get_prev_byte_count( sp->str + sp->position );
+            if ( sp->position < 0 )
+            {
+                M_err( "handle_movement", "Something's wrong here!" );
+                sp->position = 0;
+            }
+        }
 
         if ( sp->str[ sp->position ] == '\n' )
         {
@@ -632,9 +639,10 @@ handle_movement( FL_OBJECT * obj,
             if ( sp->str[ sp->position ] == '\n' )
             {
                 sp->ypos++;
-                startpos = sp->position + 1;
+                startpos = ++sp->position;
             }
-            sp->position++;
+            else
+                sp->position += utf8_get_byte_count( sp->str + sp->position );
         }
     }
     else if ( IsUp( key ) )     /* Up key */
@@ -760,16 +768,11 @@ handle_movement( FL_OBJECT * obj,
         while ( i < slen && sp->str[ i ] != ' ' && sp->str[ i ] != '\n' )
             i++;
 #else
-        if (    ! isalnum( ( unsigned char ) sp->str[ i ] )
-             && sp->str[ i ] != '_' )
-            while (    i < slen
-                    && ! isalnum( ( unsigned char ) sp->str[ i ] )
-                    && sp->str[ i ] != '_' )
+        if ( DELIM_NEW( sp->str[ i ] )  )
+            while ( i < slen && DELIM_NEW( sp->str[ i ] ) )
                 ++i;
         else
-            while (    i < slen
-                    && (    isalnum( ( unsigned char ) sp->str[ i ] )
-                         || sp->str[ i ] == '_' ) )
+            while ( i < slen && ! DELIM_NEW( sp->str[ i ] ) )
                 ++i;
 #endif
         sp->position = i;
@@ -783,7 +786,7 @@ handle_movement( FL_OBJECT * obj,
 
 static int
 handle_edit( FL_OBJECT * obj,
-             int         key,
+             FL_Char     key,
              int         slen )
 {
     FLI_INPUT_SPEC *sp = obj->spec;
@@ -795,7 +798,7 @@ handle_edit( FL_OBJECT * obj,
         if ( sp->endrange >= 0 )
             delete_piece( obj, sp->beginrange, sp->endrange - 1 );
         else if ( sp->position > 0 )
-            delete_char( sp, -1, slen );
+            delete_char( sp, -1 );
         else
             ret = FL_RETURN_NONE;
     }
@@ -804,7 +807,7 @@ handle_edit( FL_OBJECT * obj,
         if ( sp->endrange >= 0 )
             delete_piece( obj, sp->beginrange, sp->endrange - 1 );
         else if ( sp->position < slen )
-            delete_char( sp, 1, slen );
+            delete_char( sp, 1 );
         else
             ret = FL_RETURN_NONE;
     }
@@ -827,16 +830,11 @@ handle_edit( FL_OBJECT * obj,
                underscores. This is the same behaviour as in e.g. Qt.
             */
 
-            if (    ! isalnum( ( unsigned char ) sp->str[ i ] )
-                 && sp->str[ i ] != '_' )
-                while (    i < slen
-                        && ! isalnum( ( unsigned char ) sp->str[ i ] )
-                        && sp->str[ i ] != '_' )
+            if ( DELIM_NEW( sp->str[ i ] ) )
+                while ( i < slen && ! DELIM_NEW( sp->str[ i ] ) )
                     i++;
             else
-                while (    i < slen
-                        && (    isalnum( ( unsigned char ) sp->str[ i ] )
-                             || sp->str[ i ] == '_' ) )
+                while ( i < slen && DELIM_NEW( sp->str[ i ] ) )
                     i++;
 #endif
             if ( i - sp->position > 1 )
@@ -953,7 +951,7 @@ handle_edit( FL_OBJECT * obj,
     }
 #endif
     else if ( key == kmap.paste )
-        paste_it( obj, ( unsigned char * ) cutbuf, strlen( cutbuf ) );
+        paste_it( obj, cutbuf, strlen( cutbuf ) );
     else if ( key == kmap.transpose && sp->position > 0 )
     {
         char t;
@@ -983,19 +981,35 @@ handle_edit( FL_OBJECT * obj,
 
 static int
 handle_normal_key( FL_OBJECT    * obj,
-                   int            key,
+                   FL_Char        key,
                    int            slen )
 {
     FLI_INPUT_SPEC *sp = obj->spec;
     char *tmpbuf = NULL;
     int tmppos = 0;
+    int len;
     int ret = FL_RETURN_CHANGED;
 
+
+    /* We might receive a '\n' even for non-mulitline inputs (when the
+       input comes via cut-and-paste), don't accept it */
+
+    if ( key == '\n' && obj->type != FL_MULTILINE_INPUT )
+        return FL_RETURN_NONE;
+
     /* Check that there's still room for a new character */
 
+    len = utf8_get_char_bytes( key );
+    if ( len <= 0 )
+        return FL_RETURN_NONE;
+
+    /* Check if the character can be inserted without clashing with
+       the maxchar setting */
+
     if (    sp->maxchars > 0
-         && slen >= sp->maxchars
+         && slen + len > sp->maxchars
          && (    Input_Mode == FL_NORMAL_INPUT_MODE
+              || len > sp->maxchars
               || slen == sp->position ) )
     {
         fl_ringbell( 0 );
@@ -1022,14 +1036,24 @@ handle_normal_key( FL_OBJECT    * obj,
         && sp->maxchars > 0
         && slen == sp->maxchars )
     {
-        memmove( sp->str + sp->position + 1, sp->str + sp->position,
-                 slen - sp->position );
-        sp->str[ sp->maxchars ] = '\0';
+        while ( slen + len > sp->maxchars )
+        {
+            int cl = utf8_get_prev_byte_count( sp->str + slen );
+
+            if ( cl == -1 )
+                cl = 1;
+            slen -= cl;
+            sp->str[ slen ] = '\0';
+        }
+
+        memmove( sp->str + sp->position + len, sp->str + sp->position,
+                 slen - sp->position + 1 );
     }
     else
-        memmove( sp->str + sp->position + 1, sp->str + sp->position,
+        memmove( sp->str + sp->position + len, sp->str + sp->position,
                  slen - sp->position + 1 );
-    sp->str[ sp->position++ ] = key;
+
+    sp->position += utf8_insert( key, sp->str + sp->position );
 
     if ( key == '\n' )
     {
@@ -1071,7 +1095,7 @@ handle_normal_key( FL_OBJECT    * obj,
 
 static int
 handle_key( FL_OBJECT    * obj,
-            int            key,
+            FL_Char        key,
             unsigned int   kmask )
 {
     int ret = FL_RETURN_NONE;
@@ -1082,14 +1106,24 @@ handle_key( FL_OBJECT    * obj,
     int oldx = sp->xoffset;
     int oldmax = sp->max_pixels;
 
-    /* Increase the size of the buffer for the text if it's full */
+    /* Increase the size of the buffer for the text if it's getting full.
+       Decrease its size if the number of occupied bytes has dropped
+       below half of that available in the buffer. */
 
     slen = strlen( sp->str );
 
-    if ( sp->size == slen + 1 )
+    if ( sp->maxchars == 0 )
     {
-        sp->size += 8;
-        sp->str = fl_realloc( sp->str, sp->size );
+        if ( sp->size <= slen + 4 )
+        {
+            sp->size += 8;
+            sp->str  = fl_realloc( sp->str, sp->size );
+        }
+        else if ( sp->size > 2 * slen )
+        {
+            sp->size = slen + 8;
+            sp->str  = fl_realloc( sp->str, sp->size );
+        }
     }
 
     /* Silently translate carriage return to line feed */
@@ -1241,14 +1275,29 @@ handle_key( FL_OBJECT    * obj,
  ***************************************/
 
 static int
-paste_it( FL_OBJECT           * obj,
-          const unsigned char * thebytes,
-          int                   nb )
+paste_it( FL_OBJECT  * obj,
+          const char * buf,
+          int          nb )
 {
     int ret = FL_RETURN_NONE;
+    int len;
 
-    while ( nb-- )
-        ret |= handle_key( obj, *thebytes++, 0 );
+    while ( nb > 0 )
+    {
+        len = utf8_get_byte_count( buf );
+
+        if ( len == -1 )
+        {
+            buf++;
+            nb--;
+        }
+        else
+        {
+            ret |= handle_key( obj, utf8_to_num( buf ), 0 );
+            buf += len;
+            nb  -= len;
+        }
+    }
 
     return ret;
 }
@@ -1272,7 +1321,7 @@ gotit_cb( FL_OBJECT  * obj,
 {
     FLI_INPUT_SPEC *sp = obj->spec;
 
-    sp->changed |= paste_it( obj, ( unsigned char * ) buf, nb );
+    sp->changed |= paste_it( obj, buf, nb );
     fl_update_display( 0 );
 
     if ( selection_hack && sp->changed )
@@ -1362,7 +1411,7 @@ handle_input( FL_OBJECT * obj,
               int         event,
               FL_Coord    mx,
               FL_Coord    my,
-              int         key,
+              FL_Char     key,
               void      * ev )
 {
     FLI_INPUT_SPEC *sp = obj->spec;
@@ -1473,7 +1522,6 @@ handle_input( FL_OBJECT * obj,
             if ( ev )
                 ret =   ( sp->changed ? FL_RETURN_CHANGED : FL_RETURN_NONE )
                       | FL_RETURN_END;
-
             break;
 
         case FL_UPDATE:
@@ -1604,7 +1652,7 @@ input_pre( FL_OBJECT * obj,
            int         ev,
            FL_Coord    mx,
            FL_Coord    my,
-           int         key,
+           FL_Char     key,
            void      * xev )
 {
     FL_OBJECT *ext = obj->parent;
@@ -1622,7 +1670,7 @@ input_post( FL_OBJECT * obj,
             int         ev,
             FL_Coord    mx,
             FL_Coord    my,
-            int         key,
+            FL_Char     key,
             void      * xev )
 {
     FL_OBJECT *ext = obj->parent;
@@ -1684,26 +1732,31 @@ fl_create_input( int          type,
     sp->curscol        = FLI_INPUT_CCOL;
     sp->position       = -1;
     sp->endrange       = -1;
-    sp->size           = 8;
     sp->lines          = sp->ypos = 1;
-    sp->str            = fl_malloc( sp->size );
-    *sp->str           = '\0';
     sp->cursor_visible = 1;
 
     switch ( obj->type )
     {
         case FL_DATE_INPUT :
             sp->maxchars = 10;
+            sp->size     = sp->maxchars + 1;
+            sp->str      = fl_malloc( sp->size );
             break;
 
         case FL_SECRET_INPUT :
             sp->maxchars = 16;
+            sp->size     = sp->maxchars + 1;
+            sp->str      = fl_malloc( sp->size );
             break;
 
         default :
+            sp->size     = 8;
+            sp->str      = fl_malloc( sp->size );
             sp->maxchars = 0;
     }
 
+    *sp->str           = '\0';
+
     sp->dummy      = obj;
     sp->input      = obj;
     sp->field_char = ' ';
@@ -1733,7 +1786,7 @@ fake_handle( FL_OBJECT * obj,
              int         event,
              FL_Coord    mx   FL_UNUSED_ARG,
              FL_Coord    my   FL_UNUSED_ARG,
-             int         key  FL_UNUSED_ARG,
+             FL_Char     key  FL_UNUSED_ARG,
              void      * ev   FL_UNUSED_ARG )
 {
     FLI_INPUT_SPEC *sp = obj->spec;
@@ -1829,40 +1882,79 @@ fl_add_input( int          type,
 }
 
 
+
+/***************************************
+ * Function for testing if a character is suitable for an input field.
+ * These are all UTF-8 characters except non-printable characters
+ * in the range below 0x7F. For multiline inputs also a line feed
+ * is ok. Returns the number of bytes occupied by a valid character
+ * and -1 for invalid ones.
+ ***************************************/
+
+int
+is_valid( FL_OBJECT  * obj,
+          const char * p )
+{
+    int len = utf8_get_byte_count( p );
+
+    if ( len == -1 )
+        return -1;
+
+    if ( len == 1
+         && ! (    isprint( ( unsigned char ) ( *p ) )
+                || ( obj->type == FL_MULTILINE_INPUT && ( *p ) == '\n' ) ) )
+        return -1;
+
+    return len;
+}
+
+
 /***************************************
  * Sets the input string. Only printable character and, for multi-line
  * inputs new-lines are accepted.
  ***************************************/
 
-#define IS_VALID_INPUT_CHAR( c )   \
-       isprint( ( unsigned char ) ( c ) )    \
-    || ( obj->type == FL_MULTILINE_INPUT && ( c ) == '\n' )
-
 void
 fl_set_input( FL_OBJECT  * obj,
               const char * str )
 {
     FLI_INPUT_SPEC *sp = obj->spec;
     int len;
+    int cl;
     char *p;
     const char *q;
 
     if ( ! str )
         str = "";
 
-    for ( len = 0, q = str; *q; ++q )
-        if ( IS_VALID_INPUT_CHAR( *q ) )
-            ++len;
+    /* Check out how many bytes we need when skipping all non-valid
+       characters */
 
-    if ( sp->size < len + 1 )
+    for ( len = 0, q = str; *q; q += cl )
+        if ( ( cl = is_valid( obj, q ) ) == -1 )
+            cl = 1;
+        else
+            len += cl;
+
+    /* Re-allocate memory if necessary (with a bit extra)x */
+
+    if ( sp->size < len + 1 || sp->size > len + 9 )
     {
         sp->size = len + 9;
         sp->str = fl_realloc( sp->str, sp->size );
     }
 
-    for ( p = sp->str, q = str; *q; ++q )
-        if ( IS_VALID_INPUT_CHAR( *q ) )
-            *p++ = *q;
+    /* Copy all valid characters */
+
+    for ( p = sp->str, q = str; *q; q += cl )
+        if ( ( cl = is_valid( obj, q ) ) == -1 )
+            cl = 1;
+        else
+        {
+            memcpy( p, q, cl );
+            p += cl;
+        }
+
     *p = '\0';
 
     /* Set position of cursor in string to the end (if object doesn't has
@@ -2285,7 +2377,44 @@ void
 fl_set_input_maxchars( FL_OBJECT * obj,
                        int         maxchars )
 {
-    ( ( FLI_INPUT_SPEC * ) obj->spec )->maxchars = maxchars;
+    FLI_INPUT_SPEC * sp = obj->spec;
+    int len = strlen( sp->str );
+
+    if ( maxchars < 0 )
+    {
+        M_err( "fl_set_input_maxchars", "Invalid negative argument" );
+        return;
+    }
+
+    sp->maxchars = maxchars;
+
+    if ( maxchars == 0 )
+        return;
+
+    if ( sp->size < maxchars )
+    {
+        sp->str = fl_realloc( sp->str, maxchars + 1 );
+        sp->size = maxchars + 1;
+    }
+    else if ( sp->size > maxchars )
+    {
+        /* Remove as many characters from the end as necessary to make
+           the length of the string not larger than the new maximum value */
+
+        while ( len > maxchars )
+        {
+            int cl = utf8_get_prev_byte_count( sp->str + len );
+            if ( cl == -1 )
+                cl = 1;
+            len -= cl;
+            sp->str[ len ] = '\0';
+        }
+
+        sp->str = fl_realloc( sp->str, maxchars + 1 );
+        sp->size = maxchars + 1;
+
+        fl_redraw_object( obj );
+    }
 }
 
 
