@@ -231,7 +231,7 @@ handle_menu( FL_OBJECT * ob,
              int         event,
              FL_Coord    mx   FL_UNUSED_ARG,
              FL_Coord    my   FL_UNUSED_ARG,
-             int         key  FL_UNUSED_ARG,
+             FL_Char     key  FL_UNUSED_ARG,
              void      * ev   FL_UNUSED_ARG )
 {
     FLI_MENU_SPEC *sp = ob->spec;
