@@ -395,7 +395,7 @@ handle_pixmap( FL_OBJECT * obj,
                int         event,
                FL_Coord    mx   FL_UNUSED_ARG,
                FL_Coord    my   FL_UNUSED_ARG,
-               int         key  FL_UNUSED_ARG,
+               FL_Char     key  FL_UNUSED_ARG,
                void      * ev   FL_UNUSED_ARG )
 {
 #if FL_DEBUG >= ML_DEBUG
