@@ -119,7 +119,7 @@ handle_spinner( FL_OBJECT * obj,
                 int         event,
                 FL_Coord    mx   FL_UNUSED_ARG,
                 FL_Coord    my   FL_UNUSED_ARG,
-                int         key  FL_UNUSED_ARG,
+                FL_Char     key  FL_UNUSED_ARG,
                 void      * ev   FL_UNUSED_ARG )
 {
     FLI_SPINNER_SPEC *sp = obj->spec;
