@@ -113,7 +113,7 @@ handle_timer( FL_OBJECT * ob,
               int         event,
               FL_Coord    mx   FL_UNUSED_ARG,
               FL_Coord    my   FL_UNUSED_ARG,
-              int         key  FL_UNUSED_ARG,
+              FL_Char     key  FL_UNUSED_ARG,
               void      * ev   FL_UNUSED_ARG )
 {
     FLI_TIMER_SPEC *sp = ob->spec;
