@@ -66,6 +66,16 @@ draw_string( Display        * display,
 #else
 /* type fitting both XDrawString() and XDrawImageString() */
 
+#if defined X_HAVE_UTF8_STRING
+typedef void ( * DrawString )( Display    * display,
+                               Drawable     d,
+                               XFontSet     font_set,
+                               GC           gc,
+                               int          x,
+                               int          y,
+                               const char * string,
+                               int          length );
+#else
 typedef int ( * DrawString )( Display    * display,
                               Drawable     d,
                               GC           gc,
@@ -74,6 +84,7 @@ typedef int ( * DrawString )( Display    * display,
                               const char * string,
                               int          length );
 #endif
+#endif
 
 #define NUM_LINES_INCREMENT  64
 
@@ -190,8 +201,12 @@ fli_draw_string( int           align,
          * p = NULL;
 #if FL_ENABLE_XFT
     XftFont * font = fl_get_font_struct( style, size );
+#else
+#if defined X_HAVE_UTF8_STRING
+    DrawString drawIt = img ? Xutf8DrawImageString : Xutf8DrawString;
 #else
     DrawString drawIt = img ? XDrawImageString : XDrawString;
+#endif
 #endif
 
     /* Check if anything has to be drawn at all - do nothing if we either
@@ -372,9 +387,14 @@ fli_draw_string( int           align,
 #if FL_ENABLE_XFT
         draw_string( flx->display, font, forecol, line->x, line->y,
                      ( const XftChar8 * ) line->str, line->len, img );
+#else
+#if defined X_HAVE_UTF8_STRING
+        drawIt( flx->display, flx->win, fl_get_font_set( style, size ),
+                flx->textgc, line->x, line->y, line->str, line->len );
 #else
         drawIt( flx->display, flx->win, flx->textgc,
                 line->x, line->y, line->str, line->len );
+#endif
 #endif
 
         /* Draw selection area if required - for this we need to draw
@@ -427,8 +447,13 @@ fli_draw_string( int           align,
                          ( const XftChar8 * ) ( line->str + start ), len, img );
 #else
             fli_textcolor( backcol );
+#if defined X_HAVE_UTF8_STRING
+            drawIt( flx->display, flx->win,  fl_get_font_set( style, size ),
+                    flx->textgc, xsel, line->y, line->str + start, len );
+#else
             drawIt( flx->display, flx->win, flx->textgc, xsel,
                     line->y, line->str + start, len );
+#endif
 #endif
             fli_textcolor( forecol );
 
@@ -878,28 +903,32 @@ fli_draw_text_inside( int          align,
     /* Take care of special effects stuff  */
 
     if ( special == FL_SHADOW_STYLE )
-#if FL_ENABLE_XFT
-        D( x + 2, y + 2, FL_RIGHT_BCOL );
-#else
-        D( x + 2, y + 2, FL_BOTTOM_BCOL );
-#endif
+    {
+        int e = size / 18 + 2;
+
+        D( x + e, y + e, FL_RIGHT_BCOL );
+    }
     else if ( special == FL_ENGRAVED_STYLE )
     {
-        D( x - 1, y,     FL_RIGHT_BCOL );
-        D( x,     y - 1, FL_RIGHT_BCOL );
-        D( x - 1, y - 1, FL_RIGHT_BCOL );
-        D( x + 1, y,     FL_TOP_BCOL );
-        D( x,     y + 1, FL_TOP_BCOL );
-        D( x + 1, y + 1, FL_TOP_BCOL );
+        int e = size / 36 + 1;
+
+        D( x - e, y,     FL_RIGHT_BCOL );
+        D( x,     y - e, FL_RIGHT_BCOL );
+        D( x - e, y - e, FL_RIGHT_BCOL );
+        D( x + e, y,     FL_TOP_BCOL );
+        D( x,     y + e, FL_TOP_BCOL );
+        D( x + e, y + e, FL_TOP_BCOL );
     }
     else if ( special == FL_EMBOSSED_STYLE )
     {
-        D( x - 1, y,     FL_TOP_BCOL );
-        D( x,     y - 1, FL_TOP_BCOL );
-        D( x - 1, y - 1, FL_TOP_BCOL );
-        D( x + 1, y,     FL_RIGHT_BCOL );
-        D( x,     y + 1, FL_RIGHT_BCOL );
-        D( x + 1, y + 1, FL_RIGHT_BCOL );
+        int e = size / 36 + 1;
+
+        D( x - e, y,     FL_TOP_BCOL );
+        D( x,     y - e, FL_TOP_BCOL );
+        D( x - e, y - e, FL_TOP_BCOL );
+        D( x + e, y,     FL_RIGHT_BCOL );
+        D( x,     y + e, FL_RIGHT_BCOL );
+        D( x + e, y + e, FL_RIGHT_BCOL );
     }
 
     fli_draw_text_cursor( align, x, y, w, h, str, style, size,
@@ -1024,11 +1053,12 @@ fli_get_underline_rect(
 #if FL_ENABLE_XFT
     XGlyphInfo extents;
 
-    XftTextExtents8( fl_display, fs, ( const XftChar8 * ) "_", 1, &extents );
+    XftTextExtentsUtf8( fl_display, fs, ( const XftChar8 * ) "_", 1,
+                        &extents );
     ul_thickness = extents.height;
 
-    XftTextExtents8( fl_display, fs,
-                     ( const XftChar8 * ) str + n, 1, &extents );
+    XftTextExtentsUtf8( fl_display, fs,
+                        ( const XftChar8 * ) str + n, 1, &extents );
     ul_pos = extents.height - extents.y + ul_thickness / 4 + 1;
 #else
     if ( UL_thickness < 0 )
@@ -1110,15 +1140,18 @@ do_underline_all( FL_Coord        x,
     int ul_width;
 
     if ( flx->win == None )
+    {
+        *ul_pos = *ul_thickness = 0;
         return;
+    }
 
     if ( UL_thickness < 0 )
 #if FL_ENABLE_XFT
     {
         XGlyphInfo extents;
 
-        XftTextExtents8( fl_display, flx->fs, ( const XftChar8 * ) "_", 1,
-                         &extents );
+        XftTextExtentsUtf8( fl_display, flx->fs, ( const XftChar8 * ) "_", 1,
+                            &extents );
         *ul_thickness = extents.height;
     }
 #else
@@ -1133,8 +1166,8 @@ do_underline_all( FL_Coord        x,
 #if FL_ENABLE_XFT
     {
         XGlyphInfo extents;
-        XftTextExtents8( fl_display, flx->fs,
-                         ( const XftChar8 * ) str, n, &extents );
+        XftTextExtentsUtf8( fl_display, flx->fs,
+                            ( const XftChar8 * ) str, n, &extents );
         *ul_pos = extents.height - extents.y + *ul_thickness / 4 + 1;
     }
 #else
@@ -1175,7 +1208,11 @@ fli_draw_stringTAB( Drawable     win,
 #else
     GC gc;
     XFontStruct * fs;
+#if defined X_HAVE_UTF8_STRING
+    DrawString drawIt = img ? Xutf8DrawImageString : Xutf8DrawString;
+#else
     DrawString drawIt = img ? XDrawImageString : XDrawString;
+#endif
     FL_COORD cx, cy, cw, ch;
 #endif
 
@@ -1206,8 +1243,13 @@ fli_draw_stringTAB( Drawable     win,
 #if FL_ENABLE_XFT
         draw_string( flx->display, fs, color, x + w, y,
                      ( const XftChar8 * ) q, s - q, img );
+#else
+#if defined X_HAVE_UTF8_STRING
+        drawIt( flx->display, win,  fl_get_font_set( style, size ),
+                gc, x + w, y, ( char * ) q, p - q );
 #else
         drawIt( flx->display, win, gc, x + w, y, ( char * ) q, p - q );
+#endif
 #endif
         w = ( ( w + fli_get_string_width( fs, q, p - q ) ) / tab + 1 ) * tab;
     }
@@ -1215,8 +1257,13 @@ fli_draw_stringTAB( Drawable     win,
 #if FL_ENABLE_XFT
     draw_string( flx->display, fs, color, x + w, y,
                  ( const XftChar8 * ) q, s - q + len, img );
+#else
+#if defined X_HAVE_UTF8_STRING
+    drawIt( flx->display, win,  fl_get_font_set( style, size ), gc,
+            x + w, y, ( char * ) q, s - q + len );
 #else
     drawIt( flx->display, win, gc, x + w, y, ( char * ) q, s - q + len );
+#endif
 
     if ( cw > 0 || ch > 0 )
         fl_unset_gc_clipping( gc );
@@ -1246,7 +1293,7 @@ draw_string( Display        * display,
     {
         XGlyphInfo extents;
 
-        XftTextExtents8( display, font, str, len, &extents );
+        XftTextExtentsUtf8( display, font, str, len, &extents );
 
         XftDrawRect( flx->bgdraw, &flx->bktextcolor, x, y,
                      extents.width, font->height );
@@ -1254,7 +1301,7 @@ draw_string( Display        * display,
 
     fli_textcolor( fg_color );
 
-    XftDrawString8( flx->textdraw, &flx->textcolor, font, x, y, str, len );
+    XftDrawStringUtf8( flx->textdraw, &flx->textcolor, font, x, y, str, len );
 }
 #endif
 
